#include <nds.h>
#include <stdio.h>
#include <stdlib.h>

#include <maxmod9.h>
#include "soundbank_bin.h"
#include "soundbank.h"

void ClearScreen(void){
     int i;
     for(i = 0; i < 256 * 192; i++)
           VRAM_A[i] = RGB15(0,0,0);
}
void pset(unsigned char x,unsigned char y,int color){ //plots a single pixel onscreen
	if(y>=192){return;}
	VRAM_A[x%256+y*256] = color;
}
void pbox(unsigned char x1,unsigned char y1,unsigned char x2,unsigned char y2,int color){ //draws a box
	int tx=x1;
	int ty=y1;
	while(ty<=y2){
		pset(tx,ty,color);
		tx++;
		if(tx>x2){tx=x1;ty++;}
	}
}
void pframe(unsigned char x1,unsigned char y1,unsigned char x2,unsigned char y2,int color){ //draws a box
	for(int j=y1;j<=y2;j++){
		pset(x1,j,color);
		pset(x2,j,color);
	}
	for(int j=x1;j<=x2;j++){
		pset(j,y1,color);
		pset(j,y2,color);
	}
}

//tetris.c - Wilson Tetris for TRS-80 - 05/28/2019
#define pfsizeX 10 //playfield size (in cells)
#define pfsizeY 22 //note that top 4 lines are not drawn
#define pfX 2 //upper left corner of playfield location (pixel)
#define pfY 2
#define npX 128 //upper left corner of next piece indicator
#define npY 2
#define clspX 10 //spacing between cell elements (pixels)
#define clspY 10

void controls(void);

unsigned char playfield[pfsizeX][pfsizeY];  //the game area
unsigned char pieceC[4][4]; //piece container
signed char pieceCX;  //location of upper left corner of piece container
signed char pieceCY;
unsigned char pieceT; //type of piece
unsigned char pieceR; //rotation of piece 
unsigned char nextpieceT; 
unsigned char nextpieceR; 

unsigned char dcontrol; //locks in control every frame
unsigned char rcontrol;

unsigned char lastdcontrol=0; //for "debouncing" of inputs
unsigned char lastrcontrol=0; 
unsigned char drepeatframe=0;
#define drepeatframes 5 //wait _ frames before repeatedly going in one direction
bool lastPieceDown=0; //if there is a new piece and the down key is still pressed, DONT move down

unsigned char dropframe=0;    //counter for number of frames between block drops
unsigned char level=0; //LEVEL decreases frame drop from maxlevel frames to 0 frames (levels 0 to MAXLEVEL)
#define maxlevel 30

unsigned char ngame=1; //when set, starts new game
unsigned char newpc=0; //when set, get a new piece

unsigned int lines=0; //NUMBER OF LINES CLEARED
long score=0; //TOTAL SCORE (using NES rules)

unsigned char fdrop;   //number of blocks that piece has been fast dropped
unsigned char lslvi=0; //lines since level increase (when this gets to 10, increase the level)

char lvladd=0;

int tcolor(int color){ //converts from cell matrix color to NDS screen color
    int ncolor=0;
	if(color>8){color-=8;} //subtract 8 for inactive pieces colors
	switch(color){
		case 0: ncolor=RGB15(0,0,0); break; //background (no piece)
		case 1: ncolor=RGB15(0,255,255); break; //long - teal
		case 2: ncolor=RGB15(5,5,255); break; //reverse L - blue
		case 3: ncolor=RGB15(255,200,0); break; //L - orange
		case 4: ncolor=RGB15(0,255,0); break; //S - green
		case 5: ncolor=RGB15(255,0,255); break; //T - purple
		case 6: ncolor=RGB15(255,0,0); break; //Z - red
		case 7: ncolor=RGB15(255,255,0); break; //Block - yellow
		default: ncolor=RGB15(255,255,255); break; //white otherwise
	}
	return ncolor;
}
void cdraw(char x, char y, char color){//draws on screen (within playfield) (uses cell coordinates)
	int ncolor=tcolor(color); //convert cell matrix color to NDS color
	if(color==0&&x%2==0){
		ncolor=RGB15(8,8,10); //use grey instead for even cols
	}
	pbox(pfX+x*clspX,pfY+y*clspY,pfX+(x+1)*clspX-1,pfY+(y+1)*clspY-1,ncolor);
}

void recopyscreen(){ //copies playfield to screen 
  for(int i=0; i<pfsizeX; i++){ //manually redraw the entire screen
  for(int j=4; j<pfsizeY; j++){
	  cdraw(i,j-4,playfield[i][j]);
  }}
  iprintf("\x1b[3;7H%ld",score);
  iprintf("\x1b[5;7H%d",level);
  iprintf("\x1b[7;7H%d",lines);
}
void controls(){
  scanKeys();
  unsigned int keys = keysHeld();
  if(keys & KEY_UP)   {pset(250,189,RGB15(0,255,0));  }else{pset(250,189,0);}
  if(keys & KEY_LEFT) {pset(249,190,RGB15(0,255,0));  }else{pset(249,190,0);}
  if(keys & KEY_RIGHT){pset(251,190,RGB15(0,255,0));  }else{pset(251,190,0);}
  if(keys & KEY_DOWN) {pset(250,191,RGB15(0,255,0));  }else{pset(250,191,0);}
  if(keys & KEY_A)    {pset(255,191,RGB15(255,0,0));  }else{pset(255,191,0);}
  if(keys & KEY_B)    {pset(254,191,RGB15(255,255,0));}else{pset(254,191,0);}
  if(keys & KEY_START){pset(254,190,RGB15(255,255,0));ngame=1;}else{pset(254,190,0);}
  
  //if(bpeek(0x3808)&0b00000010){newpc=1;}
  if(dcontrol==0){
    if(keys & KEY_LEFT){dcontrol=4;}
    if(keys & KEY_RIGHT){dcontrol=6;}
    if(keys & KEY_UP){dcontrol=8;}
	if(!(keys & KEY_DOWN)){lastPieceDown=0;}
    if((keys & KEY_DOWN)&&lastPieceDown==0){dcontrol=2;}
    if(dcontrol==0){drepeatframe=0;}
    if(dcontrol==lastdcontrol){          //short delay before fast motion
      if(drepeatframe<=drepeatframes){
        drepeatframe++;
        dcontrol=3; //lockout if within lockout period
      }
    }
  }
  if(rcontrol==0){
    if(keys & KEY_A){rcontrol=1;}
    if(keys & KEY_B){rcontrol=2;}
    if(rcontrol==lastrcontrol){rcontrol=3;}
  }
}
void clearControls(){
  if(rcontrol!=3){lastrcontrol=rcontrol;}
  if(dcontrol!=3){lastdcontrol=dcontrol;}
  rcontrol=0;
  dcontrol=0;
}


void loadpiece(){ //hardcoded all pieces
  switch(pieceT){
    case 1: //long one
     switch(pieceR){
      case 1:
      case 3:
        pieceC[0][0]=0; pieceC[1][0]=0; pieceC[2][0]=0; pieceC[3][0]=0;
        pieceC[0][1]=0; pieceC[1][1]=0; pieceC[2][1]=0; pieceC[3][1]=0;
        pieceC[0][2]=1; pieceC[1][2]=1; pieceC[2][2]=1; pieceC[3][2]=1;
        pieceC[0][3]=0; pieceC[1][3]=0; pieceC[2][3]=0; pieceC[3][3]=0; 
        break;
      case 2:
      case 4:
        pieceC[0][0]=0; pieceC[1][0]=0; pieceC[2][0]=1; pieceC[3][0]=0;
        pieceC[0][1]=0; pieceC[1][1]=0; pieceC[2][1]=1; pieceC[3][1]=0;
        pieceC[0][2]=0; pieceC[1][2]=0; pieceC[2][2]=1; pieceC[3][2]=0;
        pieceC[0][3]=0; pieceC[1][3]=0; pieceC[2][3]=1; pieceC[3][3]=0; 
        break;
     }
     break;
   case 2: //backwards L
     switch(pieceR){
      case 1:
        pieceC[0][0]=0; pieceC[1][0]=0; pieceC[2][0]=0; pieceC[3][0]=0;
        pieceC[0][1]=0; pieceC[1][1]=0; pieceC[2][1]=0; pieceC[3][1]=0;
        pieceC[0][2]=2; pieceC[1][2]=2; pieceC[2][2]=2; pieceC[3][2]=0;
        pieceC[0][3]=0; pieceC[1][3]=0; pieceC[2][3]=2; pieceC[3][3]=0;
        break;
      case 2:
        pieceC[0][0]=0; pieceC[1][0]=0; pieceC[2][0]=0; pieceC[3][0]=0;
        pieceC[0][1]=0; pieceC[1][1]=2; pieceC[2][1]=0; pieceC[3][1]=0;
        pieceC[0][2]=0; pieceC[1][2]=2; pieceC[2][2]=0; pieceC[3][2]=0;
        pieceC[0][3]=2; pieceC[1][3]=2; pieceC[2][3]=0; pieceC[3][3]=0;
        break;
      case 3:
        pieceC[0][0]=0; pieceC[1][0]=0; pieceC[2][0]=0; pieceC[3][0]=0;
        pieceC[0][1]=2; pieceC[1][1]=0; pieceC[2][1]=0; pieceC[3][1]=0;
        pieceC[0][2]=2; pieceC[1][2]=2; pieceC[2][2]=2; pieceC[3][2]=0;
        pieceC[0][3]=0; pieceC[1][3]=0; pieceC[2][3]=0; pieceC[3][3]=0;
        break;
      case 4:
        pieceC[0][0]=0; pieceC[1][0]=0; pieceC[2][0]=0; pieceC[3][0]=0;
        pieceC[0][1]=0; pieceC[1][1]=2; pieceC[2][1]=2; pieceC[3][1]=0;
        pieceC[0][2]=0; pieceC[1][2]=2; pieceC[2][2]=0; pieceC[3][2]=0;
        pieceC[0][3]=0; pieceC[1][3]=2; pieceC[2][3]=0; pieceC[3][3]=0;
        break;
     }
     break;
   case 3: //L
     switch(pieceR){
      case 1:
        pieceC[0][0]=0; pieceC[1][0]=0; pieceC[2][0]=0; pieceC[3][0]=0;
        pieceC[0][1]=0; pieceC[1][1]=0; pieceC[2][1]=0; pieceC[3][1]=0;
        pieceC[0][2]=3; pieceC[1][2]=3; pieceC[2][2]=3; pieceC[3][2]=0;
        pieceC[0][3]=3; pieceC[1][3]=0; pieceC[2][3]=0; pieceC[3][3]=0;
        break;
      case 2:
        pieceC[0][0]=0; pieceC[1][0]=0; pieceC[2][0]=0; pieceC[3][0]=0;
        pieceC[0][1]=3; pieceC[1][1]=3; pieceC[2][1]=0; pieceC[3][1]=0;
        pieceC[0][2]=0; pieceC[1][2]=3; pieceC[2][2]=0; pieceC[3][2]=0;
        pieceC[0][3]=0; pieceC[1][3]=3; pieceC[2][3]=0; pieceC[3][3]=0;
        break;
      case 3:
        pieceC[0][0]=0; pieceC[1][0]=0; pieceC[2][0]=0; pieceC[3][0]=0;
        pieceC[0][1]=0; pieceC[1][1]=0; pieceC[2][1]=3; pieceC[3][1]=0;
        pieceC[0][2]=3; pieceC[1][2]=3; pieceC[2][2]=3; pieceC[3][2]=0;
        pieceC[0][3]=0; pieceC[1][3]=0; pieceC[2][3]=0; pieceC[3][3]=0;
        break;
      case 4:
        pieceC[0][0]=0; pieceC[1][0]=0; pieceC[2][0]=0; pieceC[3][0]=0;
        pieceC[0][1]=0; pieceC[1][1]=3; pieceC[2][1]=0; pieceC[3][1]=0;
        pieceC[0][2]=0; pieceC[1][2]=3; pieceC[2][2]=0; pieceC[3][2]=0;
        pieceC[0][3]=0; pieceC[1][3]=3; pieceC[2][3]=3; pieceC[3][3]=0;
        break;
     }
     break;
   case 4: //s shape
     switch(pieceR){
      case 1:
      case 3:
        pieceC[0][0]=0; pieceC[1][0]=0; pieceC[2][0]=0; pieceC[3][0]=0;
        pieceC[0][1]=0; pieceC[1][1]=0; pieceC[2][1]=0; pieceC[3][1]=0;
        pieceC[0][2]=0; pieceC[1][2]=4; pieceC[2][2]=4; pieceC[3][2]=0;
        pieceC[0][3]=4; pieceC[1][3]=4; pieceC[2][3]=0; pieceC[3][3]=0;
        break;
      case 2:
      case 4:
        pieceC[0][0]=0; pieceC[1][0]=0; pieceC[2][0]=0; pieceC[3][0]=0;
        pieceC[0][1]=0; pieceC[1][1]=4; pieceC[2][1]=0; pieceC[3][1]=0;
        pieceC[0][2]=0; pieceC[1][2]=4; pieceC[2][2]=4; pieceC[3][2]=0;
        pieceC[0][3]=0; pieceC[1][3]=0; pieceC[2][3]=4; pieceC[3][3]=0;
        break;
     }
     break;
   case 5: //T shape
     switch(pieceR){
      case 1:
        pieceC[0][0]=0; pieceC[1][0]=0; pieceC[2][0]=0; pieceC[3][0]=0;
        pieceC[0][1]=0; pieceC[1][1]=0; pieceC[2][1]=0; pieceC[3][1]=0;
        pieceC[0][2]=5; pieceC[1][2]=5; pieceC[2][2]=5; pieceC[3][2]=0;
        pieceC[0][3]=0; pieceC[1][3]=5; pieceC[2][3]=0; pieceC[3][3]=0;
        break;
      case 2:
        pieceC[0][0]=0; pieceC[1][0]=0; pieceC[2][0]=0; pieceC[3][0]=0;
        pieceC[0][1]=0; pieceC[1][1]=5; pieceC[2][1]=0; pieceC[3][1]=0;
        pieceC[0][2]=5; pieceC[1][2]=5; pieceC[2][2]=0; pieceC[3][2]=0;
        pieceC[0][3]=0; pieceC[1][3]=5; pieceC[2][3]=0; pieceC[3][3]=0;
        break;
      case 3:
        pieceC[0][0]=0; pieceC[1][0]=0; pieceC[2][0]=0; pieceC[3][0]=0;
        pieceC[0][1]=0; pieceC[1][1]=5; pieceC[2][1]=0; pieceC[3][1]=0;
        pieceC[0][2]=5; pieceC[1][2]=5; pieceC[2][2]=5; pieceC[3][2]=0;
        pieceC[0][3]=0; pieceC[1][3]=0; pieceC[2][3]=0; pieceC[3][3]=0;
        break;
      case 4:
        pieceC[0][0]=0; pieceC[1][0]=0; pieceC[2][0]=0; pieceC[3][0]=0;
        pieceC[0][1]=0; pieceC[1][1]=5; pieceC[2][1]=0; pieceC[3][1]=0;
        pieceC[0][2]=0; pieceC[1][2]=5; pieceC[2][2]=5; pieceC[3][2]=0;
        pieceC[0][3]=0; pieceC[1][3]=5; pieceC[2][3]=0; pieceC[3][3]=0;
        break;
     }
     break;
   case 6: //reverse s
     switch(pieceR){
      case 1:
      case 3:
        pieceC[0][0]=0; pieceC[1][0]=0; pieceC[2][0]=0; pieceC[3][0]=0;
        pieceC[0][1]=0; pieceC[1][1]=0; pieceC[2][1]=0; pieceC[3][1]=0;
        pieceC[0][2]=6; pieceC[1][2]=6; pieceC[2][2]=0; pieceC[3][2]=0;
        pieceC[0][3]=0; pieceC[1][3]=6; pieceC[2][3]=6; pieceC[3][3]=0;
        break;
      case 2:
      case 4:
        pieceC[0][0]=0; pieceC[1][0]=0; pieceC[2][0]=0; pieceC[3][0]=0;
        pieceC[0][1]=0; pieceC[1][1]=0; pieceC[2][1]=6; pieceC[3][1]=0;
        pieceC[0][2]=0; pieceC[1][2]=6; pieceC[2][2]=6; pieceC[3][2]=0;
        pieceC[0][3]=0; pieceC[1][3]=6; pieceC[2][3]=0; pieceC[3][3]=0;
        break;
     }
     break;
   case 7: //square
     switch(pieceR){
      default:
        pieceC[0][0]=0; pieceC[1][0]=0; pieceC[2][0]=0; pieceC[3][0]=0;
        pieceC[0][1]=0; pieceC[1][1]=7; pieceC[2][1]=7; pieceC[3][1]=0;
        pieceC[0][2]=0; pieceC[1][2]=7; pieceC[2][2]=7; pieceC[3][2]=0;
        pieceC[0][3]=0; pieceC[1][3]=0; pieceC[2][3]=0; pieceC[3][3]=0;
        break;
     }
     break;
  }
}

void nextpiece(){//generate the next piece and move PC back to the top
  unsigned char tempT=nextpieceT;
  unsigned char tempR=nextpieceR;
  nextpieceT = rand()%7 + 1;
  nextpieceR = rand()%4 + 1;
  pieceT=nextpieceT;
  pieceR=nextpieceR;
  loadpiece();
  for(int i=0; i<4; i++){
  for(int j=0; j<4; j++){
	  int ncolor=tcolor(pieceC[i][j]);
	  pbox(npX+i*clspX,npY+j*clspY,npX+(i+1)*clspX-1,npY+(j+1)*clspY-1,ncolor);
  }}
  
  pieceT=tempT;
  pieceR=tempR;
  
  pieceCY=0; //move piece carrier back to the top of screen
  pieceCX=3;
  
  scanKeys();
  unsigned int keys = keysHeld();
  if(keys & KEY_DOWN){lastPieceDown=1;} // if down key pressed, set lastpiecedown
}

void mdraw(){
  loadpiece(); //load piece into the piece carrier
  for(int i=0; i<6; i++){ //check piece carrier and one block offset outside it
  for(int j=0; j<6; j++){
	  controls();
	  if((pieceCX-1+i)>=0&&(pieceCX-1+i)<pfsizeX&&(pieceCY-1+j)>=4&&(pieceCY-1+j)<(pfsizeY)){//check if this segment be drawn on screen
		if(i>=1&&i<=4&&j>=1&&j<=4){//accessing piece carrier section here}
			if(pieceC[i-1][j-1]>0){//if piece carrier block is confirmed should be white
				cdraw(i-1+pieceCX,j-5+pieceCY,pieceC[i-1][j-1]);
			}else{
				cdraw(i-1+pieceCX,j-5+pieceCY,playfield[i-1+pieceCX][j-1+pieceCY]); //color it in with the existing playfield background
			}
		}else{
			cdraw(i-1+pieceCX,j-5+pieceCY,playfield[i-1+pieceCX][j-1+pieceCY]);
		}
	  }
  }}
}

unsigned char checkCollide(){ //move the piece carrier first, then check if anything collides
  loadpiece(); //load piece into the piece carrier
  unsigned char nonvalidity=0;
  for(int i=0; i<4; i++){ //run through all piece carrier cells
    for(int j=0; j<4; j++){
      if(pieceCX+i>=0&&pieceCX+i<pfsizeX&&pieceCY+j>=0&&pieceCY+j<pfsizeY){ //check if piece carrier segment can be drawn on screen
        if(pieceC[i][j]!=0&&playfield[i+pieceCX][j+pieceCY]>7){ //if both background and nonzero piece carrier segment collide
          nonvalidity=1;
        }
      }else{ //this segment of PC can't be drawn on the screen
        if(pieceC[i][j]!=0){ //a filled in segment would be drawn offscreen
          nonvalidity=1;
        }
      }
    }
  }
  return nonvalidity;
}

void newgamemon(){
  if(ngame==0){return;}
  iprintf("\x1b[3;7H            ");
  iprintf("\x1b[5;7H            ");
  iprintf("\x1b[7;7H            ");
  iprintf("\x1b[13;0H                       ");
  ngame=0;
  for(unsigned int i=0; i<pfsizeX; i++){   //clear any cells with active piece parts (will be written again with new pieceC
    for(unsigned int j=0; j<pfsizeY; j++){
      playfield[i][j]=0;
    }
  }
  lines=0;
  level=0;
  score=0;
  lslvi=0;
  fdrop=0;
  nextpiece();
  nextpiece();
  recopyscreen();
}

void piece2bg(){ //will copy active piece into background and check for lines and game over
  for(unsigned int i=0; i<4; i++){ //copy piece onto background
  for(unsigned int j=0; j<4; j++){
    if(pieceC[i][j]!=0){playfield[i+pieceCX][j+pieceCY]=pieceC[i][j]+8;} //copy the piece into the playfield/background 
  }}
  nextpiece();
  
  //check for line clears
  unsigned char clearline[pfsizeY];
  unsigned char clearedlines=0;
  unsigned char templines=0;
  for(unsigned int j=4; j<pfsizeY; j++){
  clearline[j]=1; //assume line is cleared
  for(unsigned int i=0; i<pfsizeX; i++){
    if(playfield[i][j]<=7){clearline[j]=0;break;} //line is not full
  }
  clearedlines+=clearline[j];
  templines+=clearline[j];
  }
  if(clearedlines>0){//breifly animate the cleared lines, then clear them
  
    for(unsigned char f=0; f<=4; f++){
    for(unsigned int j=4; j<pfsizeY; j++){
        if(clearline[j]==1){
          for(unsigned int i=0; i<pfsizeX; i++){
			  cdraw(i,j-4,100*(f%2));
          }
        }
      }
	  for(long i=0;i<=500000;i++){swiDelay(1);}
      //t_delay(2000);
    }
	
    for(unsigned int j=4; j<pfsizeY; j++){ //accutally clear the lines
       if(clearline[j]==1){
        for(unsigned char t=j; t>=4; t--){
         for(unsigned int i=0; i<pfsizeX; i++){
           playfield[i][t]=playfield[i][t-1];
         }
        }
       }
     }
   lines+=templines; //add number of lines to lines counter
   switch(templines){ //calculate score
    case 1:
      score+=40*(level+1);
      break;
    case 2:
      score+=100*(level+1);
      break;
    case 3:
      score+=300*(level+1);
      break;
    default:
      score+=1200*(level+1);
      break;
   }
   for(unsigned int i=1; i<=templines; i++){ //see if the level needs increasing
    lslvi++;
    if(lslvi>=10){lslvi=0;level++;}
    if(level>=maxlevel){level=maxlevel;}
   }
  }
  //check gameover (scan through line 3 and see if there are any non-active pieces in it)
  for(unsigned int i=0; i<pfsizeX; i++){
    if(playfield[i][3]>7){
      //*********************GAME OVER********************
      iprintf("\x1b[13;0HGame Over (press START)");
      while(ngame==0){
        controls();
      }
      newgamemon();
    }
  }
  
  recopyscreen();
	  
}

int main(void){
	//irqInit();
	//irqEnable(IRQ_VBLANK);
 
	videoSetMode(MODE_FB0);
	vramSetBankA(VRAM_A_LCD);
 
    ClearScreen();
	
	mmInitDefaultMem( (mm_addr)soundbank_bin );
	mmLoad(0);
	mmSelectMode(0);
	mmSetModuleVolume(1023);
	mmStart(0, MM_PLAY_LOOP );
	
	pframe(pfX-1, pfY-1, pfX+(pfsizeX)*clspX, pfY+(pfsizeY-4)*clspY, RGB15(255,255,255));
	recopyscreen();
	
	consoleDemoInit();
	iprintf("\x1b[2JWilson Tetris 2019");
	iprintf("\x1b[3;0HScore:");
	iprintf("\x1b[5;0HLevel:");
	iprintf("\x1b[7;0HLines:");

	ngame=1;
	newgamemon();
	while(1){ //*********************MAIN*PROGRAM*LOOP**************************
  
  //normally a frame delay
  for(long t=0;t<8000;t++){ //frame time is 10000 of these ticks
	//do other stuff here
	swiDelay(1); //delay
	controls(); //update controls
  }
  
  unsigned char droppiece=0;
  dropframe++;
  if(dropframe>=(maxlevel-level)){
    dropframe=0;
    droppiece=1;
  }
  newgamemon(); 
  
  if(lvladd!=0){
	  level+=lvladd;
	  lvladd=0;
	  recopyscreen();
  }
  
  if(newpc!=0){
	  newpc=0;
	  nextpiece();
	  recopyscreen();
  }
  
  if(rcontrol==1){
    pieceR++; if(pieceR>=5){pieceR=1;} //try to rotate piece
    if(checkCollide()){
      pieceR--; if(pieceR<=0){pieceR=4;} //undo rotation
    }
  }
  if(rcontrol==2){
    pieceR--; if(pieceR<=0){pieceR=4;}
    if(checkCollide()){
      pieceR++; if(pieceR>=5){pieceR=1;}
    }
  }
  if(dcontrol==4){
    pieceCX--; //try and see what happens if we move the piece left
    if(checkCollide()){//piece move is not valid
      pieceCX++; //take piece back
    }
  }
  if(dcontrol==6){
    pieceCX++; //try and see what happens if we move the piece left
    if(checkCollide()){//piece move is not valid
      pieceCX--; //take piece back
    }
  }
  if(dcontrol==8){
    pieceCY--; //try and see what happens if we move the piece up
    if(checkCollide()){//piece move is not valid
      pieceCY++; //take piece back
    }
  }
  if(dcontrol==2||droppiece==1){
    pieceCY++; //try and see what happens if we move the piece down
    if(!(drepeatframe<=drepeatframes)){ //is in fast mode
      fdrop++;
    }else{
      fdrop=0;
    }
    if(checkCollide()){//piece move is not valid
      pieceCY--; //take piece back
      score+=fdrop; //add # of fast dropped blocks to score
      fdrop=0;
      piece2bg(); //copy piece to background and reset to next piece
    }
  }
  
  clearControls();
  mdraw();
}
}